/************************************************************
 *  Martin Toepfer     ***     PROJECT : Lingvisto          *
 *  Wissmannstr. 5     ***                                  *
 *  D - 22041 Hamburg  ***     File    : xaw.c              *
 *  Germany            ***     Date    : 04.04.1997         *
 ************************************************************/
#include <fcntl.h>
#include <setjmp.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Repeater.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include "datastructure.h"
#include "edit.h"
#include "file.h"
#include "test.h"
#include "xaw.h"
#include "xawfunction.h"

/************************************************************
 *          global variables (in datastructure.c)           *
 ************************************************************/
extern struct lvo_file head;
extern NODE current;

/************************************************************
 *             some more global variables                   *
 ************************************************************/
XtAppContext app_con;
Boolean b;
jmp_buf env1;
jmp_buf env2;

/************************************************************
 *          some keyboard translation strings               *
 ************************************************************/
String trans1   = "<Key>Escape: no-op()";
String trans2   = "<Key>Escape: JosephExit()";
String trans3   = "<Key>Escape: InsertExit()";
String trans4   = "<Key>Return: PopdownAction()";
String trans5   = "<Key>Return: DestroyAction()";
String transFWD = "<Key>Right: ForwardAction()";
String transREV = "<Key>Left: ReverseAction()";

/************************************************************
 *                 some extern widgets                      *
 ************************************************************/
Widget top,                              /* toplevel widged */    
       form,
       native,                        /* native word widget */
       foreign,                      /* foreign word widget */
       statline,                            /* state widget */
       file,                             /* file menuWidget */
       edit,                             /* edit menuWidget */
       test,                             /* test menuWidget */
       fwd,                             /* 'forward' button */
       rev;                             /* 'reverse' button */

/************************************************************
 *                 'File' menu entries                      *
 ************************************************************/
static char *file_menu[] = {
  "New", "Load", "Save", "Save as", "Quit"
};

/**************************************************
 *       'File' menu callback function            *
 **************************************************/
/* ARGSUSED */
static void FileMenu(Widget w, XtPointer x, XtPointer y) {
  if (strcmp(XtName(w), "New") == 0)
    new_file();
  if (strcmp(XtName(w), "Load") == 0)
    load_file();
  if (strcmp(XtName(w), "Save") == 0)
    save_file();
  if (strcmp(XtName(w), "Save as") == 0)
    save_as_file();
  if (strcmp(XtName(w), "Quit") == 0)
    quit_file();
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
  DisplayState(get_filename());
}

/************************************************************
 *                 'Edit' menu entries                      *
 ************************************************************/
static char *edit_menu[] = {
  "Insert", "Change", "Find native", "Find foreign", "Delete"
};

/**************************************************
 *       'Edit' menu callback function            *
 **************************************************/
/* ARGSUSED */
static void EditMenu(Widget w, XtPointer x, XtPointer y) {
  if (strcmp(XtName(w), "Insert") == 0)
    InsertPair();
  if (strcmp(XtName(w), "Change") == 0)
    ChangePair();
  if (strcmp(XtName(w), "Find native") == 0)
    FindPair((unsigned char) NATIVE);
  if (strcmp(XtName(w), "Find foreign") == 0)
    FindPair((unsigned char) FOREIGN);
  if (strcmp(XtName(w), "Delete") == 0)
    delete_pair();
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
}

/************************************************************
 *                 'Test' menu entries                      *
 ************************************************************/
static char *test_menu[] = {
  "Native", "Native (multiple choice)",
  "Foreign", "Foreign (multiple choice)"
};

/**************************************************
 *       'Test' menu callback function            *
 **************************************************/
/* ARGSUSED */
static void TestMenu(Widget w, XtPointer x, XtPointer y) {
  if (strcmp(XtName(w), "Native") == 0)
    joseph((unsigned char) NATIVE);
  if (strcmp(XtName(w), "Native (multiple choice)") == 0)
    joseph((unsigned char) MNATIVE);
  if (strcmp(XtName(w), "Foreign") == 0)
    joseph((unsigned char) FOREIGN);
  if (strcmp(XtName(w), "Foreign (multiple choice)") == 0)
    joseph((unsigned char) MFOREIGN);
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
}

/**************************************************
 *       ' << ' button callback function          *
 **************************************************/
/* ARGSUSED */
static void Reverse(Widget w, XtPointer x, XtPointer y) {
  reverse_pair();
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
}

/**************************************************
 *       ' >> ' button callback function          *
 **************************************************/
/* ARGSUSED */
static void Forward(Widget w, XtPointer x, XtPointer y) {
  forward_pair();
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
}

/************************************************************
 *                   application actions                    *
 ************************************************************/
/**************************************************
 * pressing 'Escape' in test mode exits test      *
 **************************************************/
void JosephExit(Widget w, XEvent *e, String *p, Cardinal *n) {
  extern unsigned short known;
  extern unsigned short unknown;
  
  /*
   * disable input in asciiTextWidgets
   */
  
  XtVaSetValues(native,
		XtNeditType, XawtextRead,
		XtNdisplayCaret, False,
		NULL);
  XtVaSetValues(foreign,
		XtNeditType, XawtextRead,
		XtNdisplayCaret, False,
		NULL);
  
  /*
   * cleanup and statistics
   */
  
  restore_list();
  DisplayResult(known, unknown);
  longjmp(env2, 1);
}

/**************************************************
 * pressing 'Escape' in insert mode exits insert  *
 **************************************************/
static void InsertExit(Widget w, XEvent *e, String *p, Cardinal *n) {
  XtVaSetValues(w,
		XtNeditType, XawtextRead,
		XtNdisplayCaret, False,
		NULL);
  XtSetKeyboardFocus(top, top);
  SetAllSensitive((Boolean) True);
  DisplayState(get_filename());
  longjmp(env1, 1);
}

/**************************************************
 * pressing 'Return' in the popup shell that dis- *
 * plays the error message in test mode will      *
 * popdown the shell                              *
 **************************************************/
void Popdown(Widget w, XtPointer x, XtPointer y);
static void PopdownAction(Widget w, XEvent *e, String *p, Cardinal *n) {
  Popdown(w, NULL, NULL);
}

/**************************************************
 * pressing 'Return' in all popup shells that     *
 * contain one single 'ok' button will popdown    *
 **************************************************/
static void DestroyAction(Widget w, XEvent *e, String *p, Cardinal *n) {
  Destroy(w, NULL, NULL);
}

/**************************************************
 * pressing 'Return' in fileselector-box will     *
 * load file whose name is specified in the       *
 * input asciiTextWidget                          *
 **************************************************/
void Load(Widget w, XtPointer x, XtPointer y);
static void LoadAction(Widget w, XEvent *e, String *p, Cardinal *n) {
  Load(w, NULL, NULL);
}

/**************************************************
 * pressing the left cursor key will scroll rev.  *
 **************************************************/
static void ReverseAction(Widget w, XEvent *e, String *p, Cardinal *n) {
  Reverse(w, NULL, NULL);
}

/**************************************************
 * pressing the right cursor key will scroll fwd. *
 **************************************************/
static void ForwardAction(Widget w, XEvent *e, String *p, Cardinal *n) {
  Forward(w, NULL, NULL);
}

/**************************************************
 * this action has no effect                      *
 **************************************************/
static void Noop(Widget w, XEvent *e, String *p, Cardinal *n) {
}

/************************************************************
 * init_xaw : connect to X-server and create all widgets    *
 ************************************************************/
void init_xaw(int argc, char *argv[]) {
  Widget menu, entry, nlabel, flabel;
  int fd;                               /* file descriptor */
  unsigned char i;
  
  /*
   * these are the applications special actions
   */
  
  static XtActionsRec actions[] = {
    {"JosephExit", JosephExit},
    {"InsertExit", InsertExit},
    {"PopdownAction", PopdownAction},
    {"DestroyAction", DestroyAction},
    {"LoadAction", LoadAction},
    {"ForwardAction", ForwardAction},
    {"ReverseAction", ReverseAction},
    {"Noop", Noop}
  };
  
  /*
   * this is the toplevel widget
   */
  
  top = XtVaAppInitialize(&app_con, "Lingvisto", NULL, 0, 
			  &argc, argv, NULL, NULL);
  XtAppAddActions(app_con, actions, 8);
  XtVaSetValues(top,
		XtNtitle, "Lingvisto 1.1",
		NULL);
  
  /*
   * this is the formWidget of the main window
   */
  
  form = XtVaCreateManagedWidget("form", formWidgetClass, top, NULL);
  
  /*
   * this is the 'file' menu
   */
  
  file = XtVaCreateManagedWidget("File", menuButtonWidgetClass, form, NULL);
  XtVaSetValues(file, 
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  menu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, file, NULL);
  for (i = (unsigned char) 0; i < 5; i++) {
    entry = XtVaCreateManagedWidget(file_menu[i], smeBSBObjectClass, menu, NULL);
    XtAddCallback(entry, XtNcallback, FileMenu, NULL);
  }
  
  /*
   * this is the 'edit' menu
   */
  
  edit = XtVaCreateManagedWidget("Edit", menuButtonWidgetClass, form, NULL);
  XtVaSetValues(edit, 
		XtNfromHoriz, file, 
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  menu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, edit, NULL);
  for (i = (unsigned char) 0; i < 5; i++) {
    entry = XtVaCreateManagedWidget(edit_menu[i], smeBSBObjectClass, menu, NULL);
    XtAddCallback(entry, XtNcallback, EditMenu, NULL);
  }
  
  /*
   * this is the 'test' menu
   */
  
  test = XtVaCreateManagedWidget("Test", menuButtonWidgetClass, form, NULL);
  XtVaSetValues(test, 
		XtNfromHoriz, edit, 
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  menu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, test, NULL);
  for (i = (unsigned char) 0; i < 4; i++) {
    entry = XtVaCreateManagedWidget(test_menu[i], smeBSBObjectClass, menu, NULL);
    XtAddCallback(entry, XtNcallback, TestMenu, NULL);
  }
  
  /*
   * this is the labelWidget that contains the string 'Native :'
   */
  
  nlabel = XtVaCreateManagedWidget("Native :", labelWidgetClass, form, NULL);
  XtVaSetValues(nlabel,
		XtNfromVert, file,
		XtNvertDistance, 13,
		XtNjustify, XtJustifyLeft,
		XtNborderWidth, 0,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  
  /*
   * this is the asciiTextWidget that contains the native word
   */ 
  
  native = XtVaCreateManagedWidget("native", asciiTextWidgetClass, form, NULL);
  XtVaSetValues(native,
		XtNfromVert, nlabel,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNwidth, 388,
		XtNtype, XawAsciiString,
		XtNdisplayCaret, False,
		NULL);
  
  /*
   * this is the labelWidget that contains the string 'Foreign :'
   */
  
  flabel = XtVaCreateManagedWidget("Foreign :", labelWidgetClass, form, NULL);
  XtVaSetValues(flabel,
		XtNfromVert, native,
		XtNjustify, XtJustifyLeft,
		XtNborderWidth, 0,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  
  /*
   * this is the asciiTextWidget that contains the foreign word
   */
  
  foreign = XtVaCreateManagedWidget("foreign", asciiTextWidgetClass, form, NULL);
  XtVaSetValues(foreign,
		XtNfromVert, flabel,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNwidth, 388,
		XtNtype, XawAsciiString,
		XtNdisplayCaret, False,
		NULL);
  
  /*
   * this is the 'reverse' button
   */
  
  rev = XtVaCreateManagedWidget(" << ", repeaterWidgetClass, form, NULL);
  XtAddCallback(rev, XtNcallback, Reverse, NULL);
  XtVaSetValues(rev, 
		XtNfromVert, foreign, 
		XtNvertDistance, 12,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  
  /*
   * this is the 'forward' button
   */
  
  fwd = XtVaCreateManagedWidget(" >> ", repeaterWidgetClass, form, NULL);
  XtAddCallback(fwd, XtNcallback, Forward, NULL);
  XtVaSetValues(fwd,
		XtNfromVert, foreign,
		XtNvertDistance, 12,
		XtNfromHoriz, rev,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNleft, XawChainLeft,
		XtNright, XawChainLeft,
		NULL);
  
  /*
   * this labelWidget displays the states of the program
   */
  
  statline = XtVaCreateManagedWidget("", labelWidgetClass, form, NULL);
  XtVaSetValues(statline,
		XtNfromVert, rev,
		XtNvertDistance, 13,
		XtNtop, XawChainTop,
		XtNbottom, XawChainTop,
		XtNwidth, 386,
		XtNjustify, XtJustifyLeft,
		XtNborderWidth, 3,
		NULL);
  
  /*
   * load file if filename specified at command-line
   */
  
  set_filename(argv[argc-1]);
  if ((fd = open(get_filename(), O_RDONLY)) != -1) {
    if (read_file(fd) != 0) {
      restore_filename();
    }
  }
  else
    restore_filename();
  
  /*
   * and now lets start up the toplevel widget
   */
  
  XtRealizeWidget(top);
  XtOverrideTranslations(form, XtParseTranslationTable(transREV));
  XtOverrideTranslations(form, XtParseTranslationTable(transFWD));
  DisplayNodeNative(current);
  DisplayNodeForeign(current);
  DisplayState(get_filename());
  XtAppMainLoop(app_con);
}
